import { Survey, SurveyChoice } from "../entities/Survey";
import { RequestHandler } from "../entities/RequestHandler";
import { SurveyHelper } from "../helpers/SurveyHelper";

/**
 * Survey controller
 * 
 * @author Pierre HUBERT
 */

export class SurveyController {

	/**
	 * Send the response to a survey to the server
	 * 
	 * @param h Request handler
	 */
	public static async SendResponse(h: RequestHandler) {
		const surveyID = await this.PostSurveyIDFromPostID(h, "postID");
		const choiceID = h.postInt("choiceID");

		await SurveyHelper.CancelResponse(h.getUserId(), surveyID);

		if(!await SurveyHelper.ChoiceExists(surveyID, choiceID))
			h.error(404, "Choice not found for this survey!");
		
		await SurveyHelper.SendResponse(h.getUserId(), surveyID, choiceID);

		h.success();
	}

	/**
	 * Cancel the repsonse to a request
	 * 
	 * @param h Request handler
	 */
	public static async CancelResponse(h: RequestHandler) {
		const surveyID = await this.PostSurveyIDFromPostID(h, "postID");
		
		await SurveyHelper.CancelResponse(h.getUserId(), surveyID);

		h.success();
	}


	/**
	 * Turn a survey into an API entry
	 * 
	 * @param h Request handler
	 * @param survey The survey
	 */
	public static async SurveyToAPI(h: RequestHandler, survey: Survey) : Promise<any> {
		let data  = {
			ID: survey.id,
			userID: survey.userID,
			postID: survey.postID,
			creation_time: survey.timeCreate,
			question: survey.question,
			user_choice: -1,
			choices: {},
		}

		survey.choices.forEach((c) => data.choices[c.id.toString()] = this.SurveyChoiceToAPI(c))

		if(h.signedIn)
			data.user_choice = await SurveyHelper.GetUserChoice(survey.id, h.getUserId());

		return data;
	}

	/**
	 * Get the ID of a survey from a POST id
	 * 
	 * @param h Request handler
	 * @param field The name of the POST field containing the
	 * POST id
	 */
	private static async PostSurveyIDFromPostID(h: RequestHandler, field: string) : Promise<number> {
		const postID = await h.postPostIDWithAccess(field);
		return await SurveyHelper.GetID(postID);
	}

	/**
	 * Turn a survey choice into an API entry
	 * 
	 * @param c The choice
	 */
	private static SurveyChoiceToAPI(c: SurveyChoice) {
		return {
			choiceID: c.id,
			name: c.name,
			responses: c.count
		}
	}
}